/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/

package org.fortiss.af3.rcp.application;

import static org.eclipse.ui.PlatformUI.RETURN_RESTART;
import static org.eclipse.ui.PlatformUI.createAndRunWorkbench;
import static org.eclipse.ui.PlatformUI.createDisplay;
import static org.eclipse.ui.PlatformUI.getWorkbench;

import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbench;
import org.fortiss.af3.project.ui.perspective.ModelingPerspectiveFactory;
import org.fortiss.af3.rcp.application.advisors.ApplicationWorkbenchAdvisor;

/**
 * Application class for the AF3 RCP.
 * 
 * @author ratiu
 */
public class AF3Application implements IApplication {

	/** {@inheritDoc} */
	@Override
	public Object start(IApplicationContext context) throws Exception {
		final Display display = createDisplay();
		try {
			// Switch to modeling perspective on startup. This has the same effect as using the
			// -perspective flag in eclipse.ini / autofocus3-phoenix.ini
			// org.eclipse.e4.ui.internal.workbench.swt.E4Application.PERSPECTIVE_ARG_NAME
			System.setProperty("perspective", ModelingPerspectiveFactory.MODELING_PERSPECTIVE_ID);

			final int returnCode =
					createAndRunWorkbench(display, new ApplicationWorkbenchAdvisor());
			if(returnCode == RETURN_RESTART) {
				return IApplication.EXIT_RESTART;
			}
			return IApplication.EXIT_OK;
		} finally {
			display.dispose();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void stop() {
		final IWorkbench workbench = getWorkbench();
		if(workbench == null) {
			return;
		}

		final Display display = workbench.getDisplay();
		display.syncExec(new Runnable() {
			@Override
			public void run() {
				if(!display.isDisposed()) {
					workbench.close();
				}
			}
		});
	}
}
